/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2009 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.01 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_01.txt.                                 |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Johannes Schlueter                                           |
  +----------------------------------------------------------------------+
*/

/* $Id: idn.c,v 1.8 2009/01/06 14:55:07 johannes Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_idn.h"
#include "Zend/zend_API.h"

#include <stringprep.h>
#include <idna.h>

/* If you declare any globals in php_idn.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(idn)
*/

/* True global resources - no need for thread safety here */
static int le_idn;

/* {{{ arginfo */
ZEND_BEGIN_ARG_INFO_EX(arginfo_idn_to_ascii, 0, 0, 1)
	ZEND_ARG_INFO(0, utf8_domain)
	ZEND_ARG_INFO(1, errorcode)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_idn_to_utf8, 0, 0, 1)
	ZEND_ARG_INFO(0, ascii_domain)
	ZEND_ARG_INFO(1, errorcode)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_idn_strerror, 0, 0, 1)
	ZEND_ARG_INFO(0, errorcode)
ZEND_END_ARG_INFO()
/* }}} */

/* {{{ PHP_INI
 */
/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
	STD_PHP_INI_ENTRY("idn.global_value",	   "42", PHP_INI_ALL, OnUpdateLong, global_value, zend_idn_globals, idn_globals)
	STD_PHP_INI_ENTRY("idn.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_idn_globals, idn_globals)
PHP_INI_END()
*/
/* }}} */

/* {{{ php_idn_init_globals
 */
/* Uncomment this function if you have INI entries
static void php_idn_init_globals(zend_idn_globals *idn_globals)
{
	idn_globals->global_value = 0;
	idn_globals->global_string = NULL;
}
*/
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(idn)
{
	/* Check that the the version of the GNU libidn library */
	if (!stringprep_check_version(STRINGPREP_VERSION)) {
		return FAILURE;
	}
	
	REGISTER_LONG_CONSTANT("IDNA_STRINGPREP_ERROR",		  IDNA_STRINGPREP_ERROR,	   CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_PUNYCODE_ERROR",		  IDNA_PUNYCODE_ERROR,		   CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_CONTAINS_NON_LDH",		  IDNA_CONTAINS_NON_LDH,	   CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_CONTAINS_MINUS",		  IDNA_CONTAINS_MINUS,		   CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_INVALID_LENGTH",		  IDNA_INVALID_LENGTH,		   CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_NO_ACE_PREFIX",		  IDNA_NO_ACE_PREFIX,		   CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_ROUNDTRIP_VERIFY_ERROR", IDNA_ROUNDTRIP_VERIFY_ERROR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_CONTAINS_ACE_PREFIX",	  IDNA_CONTAINS_ACE_PREFIX,    CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_ICONV_ERROR",			  IDNA_ICONV_ERROR,			   CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_MALLOC_ERROR",			  IDNA_MALLOC_ERROR,		   CONST_CS | CONST_PERSISTENT);

	/* If you have INI entries, uncomment these lines 
	ZEND_INIT_MODULE_GLOBALS(idn, php_idn_init_globals, NULL);
	REGISTER_INI_ENTRIES();
	*/

	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(idn)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "GNU Libidn support", "enabled");
	php_info_print_table_row(2, "GNU Libidn version", STRINGPREP_VERSION);
	php_info_print_table_row(2, "Extension version", PHP_IDN_VERSION);
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}
/* }}} */

/* {{{ proto string idn_to_ascii(string utf8_domain[, int &errorcode])
   Converts an UTF-8 encoded string to corresponding ASCII name. */
PHP_FUNCTION(idn_to_ascii)
{
	/* The domain name the user enters */
	char *utf8_domain = NULL;
	int utf8_domain_len;

	/* PHP reference for error code */
	zval *errorcode = NULL;

	/* libidn return values */
	char *ascii_domain;
	Idna_rc rc;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|z", &utf8_domain, &utf8_domain_len, &errorcode) == FAILURE)
	{
		return;
	}

	/* UTF-8 to ASCII conversion. */
	rc = idna_to_ascii_8z(utf8_domain, &ascii_domain, 0);
	if (rc != IDNA_SUCCESS) {
		free(ascii_domain);

		if (errorcode != NULL) {
			ZVAL_LONG(errorcode, rc);
		}

		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not convert %s to ASCII: %s", utf8_domain, idna_strerror(rc));
		RETURN_FALSE;
	}

	RETVAL_STRINGL(ascii_domain, strlen(ascii_domain), 1);
	free(ascii_domain);
}
/* }}} */

/* {{{ proto string idn_to_utf8(string ascii_domain[, int &errorcode])
   Converts an ASCII compatible domain name to UTF-8. */
PHP_FUNCTION(idn_to_utf8)
{
	/* The domain name the user enters */
	char *ascii_domain = NULL;
	int ascii_domain_len;

	/* PHP reference for error code */
	zval *errorcode = NULL;

	/* libidn return values */
	char *utf8_domain = NULL;
	Idna_rc rc;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|z", &ascii_domain, &ascii_domain_len, &errorcode) == FAILURE)
	{
		return;
	}

	/* ASCII to UTF-8 conversation */
	rc = idna_to_unicode_8z8z(ascii_domain, &utf8_domain, 0);
	if (rc != IDNA_SUCCESS) {
		free(utf8_domain);

		if (errorcode != NULL) {
			ZVAL_LONG(errorcode, rc);
		}

		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not convert '%s' to UTF-8: %s", ascii_domain, idna_strerror(rc));
		RETURN_FALSE;
	}

	RETVAL_STRINGL(utf8_domain, strlen(utf8_domain), 1);
	free(utf8_domain);
}
/* }}} */

/* {{{ proto string idn_strerror(int errorcode)
   Returns corresponding error message. */
PHP_FUNCTION(idn_strerror)
{
	/* The error code the user enters */
	long errorcode;

	/* libidn return values */
	const char *errormsg;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &errorcode) == FAILURE)
	{
		return;
	}

	errormsg = idna_strerror(errorcode);
	RETURN_STRINGL((char*)errormsg, strlen(errormsg), 1);
}
/* }}} */


/* {{{ idn_functions[] */
function_entry idn_functions[] = {
	PHP_FE(idn_strerror, arginfo_idn_strerror)
	PHP_FE(idn_to_ascii, arginfo_idn_to_ascii)
	PHP_FE(idn_to_utf8,  arginfo_idn_to_utf8)
	{NULL, NULL, NULL}
};
/* }}} */

/* {{{ idn_module_entry
 */
zend_module_entry idn_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"idn",
	idn_functions,
	PHP_MINIT(idn),
	NULL,
	NULL,
	NULL,
	PHP_MINFO(idn),
#if ZEND_MODULE_API_NO >= 20010901
	PHP_IDN_VERSION,
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_IDN
ZEND_GET_MODULE(idn)
#endif


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
